<?php

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

if ( ! class_exists( 'WooThemes_Sensei_Email_Teacher_New_Message' ) ) :

/**
 * Teacher New Message
 *
 * An email sent to the teacher when one of their students sends them a private message.
 *
 * @class 		WooThemes_Sensei_Email_Teacher_New_Message
 * @version		1.6.0
 * @package		Sensei/Classes/Emails
 * @author 		WooThemes
 */
class WooThemes_Sensei_Email_Teacher_New_Message {

	var $template;
	var $subject;
	var $heading;
	var $recipient;
	var $learner;
	var $teacher;
	var $message;

	/**
	 * Constructor
	 *
	 * @access public
	 * @return void
	 */
	function __construct() {
		$this->template = 'teacher-new-message';
		$this->subject = apply_filters( 'sensei_email_subject', sprintf( __( '[%1$s] You have received a new private message', 'woothemes-sensei' ), get_bloginfo( 'name' ) ), $this->template );
		$this->heading = apply_filters( 'sensei_email_heading', __( 'Your student has sent you a private message', 'woothemes-sensei' ), $this->template );
	}

	/**
	 * trigger function.
	 *
	 * @access public
	 * @return void
	 */
	function trigger( $message_id = 0 ) {
		global $woothemes_sensei, $sensei_email_data;

		$this->message = get_post( $message_id );

		$learner_username = get_post_meta( $message_id, '_sender', true );
		$this->learner = get_user_by( 'login', $learner_username );

		$teacher_username = get_post_meta( $message_id, '_receiver', true );
		$this->teacher = get_user_by( 'login', $teacher_username );

		$content_type = get_post_meta( $message_id, '_posttype', true );
		$content_id = get_post_meta( $message_id, '_post', true );
		$content_title = get_the_title( $content_id );

		// Construct data array
		$sensei_email_data = apply_filters( 'sensei_email_data', array(
			'template'			=> $this->template,
			'heading'			=> $this->heading,
			'teacher_id'		=> $this->teacher->ID,
			'learner_id'		=> $this->learner->ID,
			'learner_name'		=> $this->learner->display_name,
			'message_id'		=> $message_id,
			'message'			=> $this->message->post_content,
			'content_title'		=> $content_title,
			'content_type'		=> $content_type,
		), $this->template );

		// Set recipient (teacher)
		$this->recipient = stripslashes( $this->teacher->user_email );

		// Send mail
		$woothemes_sensei->emails->send( $this->recipient, $this->subject, $woothemes_sensei->emails->get_content( $this->template ) );

		wp_safe_redirect( add_query_arg( array( 'send' => 'complete' ) ) );
		exit;
	}
}

endif;

return new WooThemes_Sensei_Email_Teacher_New_Message();